/// <reference types="node" />
import { Beet, BeetField, SupportedTypeDefinition } from './types';
import { CollectionsExports, CollectionsTypeMapKey } from './collections';
import { CompositesExports, CompositesTypeMapKey } from './composites';
import { NumbersExports, NumbersTypeMapKey } from './numbers';
export * from './collections';
export * from './composites';
export * from './numbers';
export * from './types';
/**
 * @category TypeDefinition
 */
export declare type BeetTypeMapKey = CollectionsTypeMapKey | CompositesTypeMapKey | NumbersTypeMapKey;
/**
 * @category TypeDefinition
 */
export declare type BeetExports = CollectionsExports | CompositesExports | NumbersExports;
/**
 * Maps all {@link Beet} de/serializers to metadata which describes in which
 * package it is defined as well as which TypeScript type is used to represent
 * the deserialized value in JavaScript.
 *
 * @category TypeDefinition
 */
export declare const supportedTypeMap: Record<BeetTypeMapKey, SupportedTypeDefinition & {
    beet: BeetExports;
}>;
/**
 * Underlying writer used to serialize structs.
 *
 * @private
 * @category beet/struct
 */
export declare class BeetWriter {
    private buf;
    private _offset;
    constructor(byteSize: number);
    get buffer(): Buffer;
    get offset(): number;
    private maybeResize;
    write<T>(beet: Beet<T>, value: T): void;
    writeStruct<T>(instance: T, fields: BeetField<T>[]): void;
}
/**
 * Underlying reader used to deserialize structs.
 *
 * @private
 * @category beet/struct
 */
export declare class BeetReader {
    private readonly buffer;
    private _offset;
    constructor(buffer: Buffer, _offset?: number);
    get offset(): number;
    read<T>(beet: Beet<T>): T;
    readStruct<T>(fields: BeetField<T>[]): T;
}
/**
 * Configures a class or any JavaScript object type for de/serialization aka
 * read/write.
 *
 * @template Class the type to produce when deserializing
 * @template Args contains all fields, is typically a subset of Class and is
 * used to construct an instance of it
 *
 * @category beet/struct
 */
export declare class BeetStruct<Class, Args = Partial<Class>> implements Beet<Class> {
    private readonly fields;
    private readonly construct;
    readonly description: string;
    readonly byteSize: number;
    /**
     * Creates an instance of the BeetStruct.
     *
     * @param fields de/serializers for each field of the {@link Class}
     * @param construct the function that creates an instance of {@link Class}
     * from the args
     * @param description identifies this struct for diagnostics/debugging
     * purposes
     */
    constructor(fields: BeetField<Args>[], construct: (args: Args) => Class, description?: string);
    /**
     * Along with `write` this allows structs to be treated as {@link Beet}s and
     * thus supports composing/nesting them the same way.
     * @private
     */
    read(buf: Buffer, offset: number): Class;
    /**
     * Along with `read` this allows structs to be treated as {@link Beet}s and
     * thus supports composing/nesting them the same way.
     * @private
     */
    write(buf: Buffer, offset: number, value: Args): void;
    /**
     * Deserializes an instance of the Class from the provided buffer starting to
     * read at the provided offset.
     *
     * @returns `[instance of Class, offset into buffer after deserialization completed]`
     */
    deserialize(buffer: Buffer, offset?: number): [Class, number];
    /**
     * Serializes the provided instance into a new {@link Buffer}
     *
     * @param instance of the struct to serialize
     * @param byteSize allows to override the size fo the created Buffer and
     * defaults to the size of the struct to serialize
     */
    serialize(instance: Args, byteSize?: number): [Buffer, number];
    private getByteSize;
    static description: string;
}
/**
 * Convenience wrapper around {@link BeetStruct} which is used for plain JavasScript
 * objects, like are used for option args passed to functions.
 *
 * @category beet/struct
 */
export declare class BeetArgsStruct<Args> extends BeetStruct<Args, Args> {
    constructor(fields: BeetField<Args>[], description?: string);
    static description: string;
}
