"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.collectionsTypeMap = exports.fixedSizeUint8Array = exports.fixedSizeBuffer = exports.fixedSizeArray = exports.fixedSizeUtf8String = void 0;
const types_1 = require("./types");
const assert_1 = require("assert");
const numbers_1 = require("./numbers");
const types_2 = require("./types");
/**
 * De/Serializes a UTF8 string of a particular size.
 *
 * @param stringByteLength the number of bytes of the string
 *
 * @category beet/collection
 */
const fixedSizeUtf8String = (stringByteLength) => {
    return {
        write: function (buf, offset, value) {
            const stringBuf = Buffer.from(value, 'utf8');
            assert_1.strict.equal(stringBuf.byteLength, stringByteLength, `${value} has invalid byte size`);
            numbers_1.u32.write(buf, offset, stringByteLength);
            stringBuf.copy(buf, offset + 4, 0, stringByteLength);
        },
        read: function (buf, offset) {
            const size = numbers_1.u32.read(buf, offset);
            assert_1.strict.equal(size, stringByteLength, `invalid byte size`);
            const stringSlice = buf.slice(offset + 4, offset + 4 + stringByteLength);
            return stringSlice.toString('utf8');
        },
        byteSize: 4 + stringByteLength,
        description: `utf8-string(${stringByteLength})`,
    };
};
exports.fixedSizeUtf8String = fixedSizeUtf8String;
/**
 * De/Serializes an array with a specific number of elements of type {@link T}.
 *
 * @template T type of elements held in the array
 *
 * @param element the De/Serializer for the element type
 * @param len the number of elements in the array
 * @param lenPrefix if `true` a 4 byte number indicating the size of the array
 * will be included before serialized array data
 *
 * @category beet/collection
 */
function fixedSizeArray(element, len, lenPrefix = false) {
    const arraySize = element.byteSize * len;
    const byteSize = lenPrefix ? 4 + arraySize : arraySize;
    return {
        write: function (buf, offset, value) {
            assert_1.strict.equal(value.length, len, `array length ${value.length} should match len ${len}`);
            if (lenPrefix) {
                numbers_1.u32.write(buf, offset, len);
                offset += 4;
            }
            for (let i = 0; i < len; i++) {
                element.write(buf, offset + i * element.byteSize, value[i]);
            }
        },
        read: function (buf, offset) {
            if (lenPrefix) {
                const size = numbers_1.u32.read(buf, offset);
                assert_1.strict.equal(size, len, 'invalid byte size');
                offset += 4;
            }
            const arr = new Array(len);
            for (let i = 0; i < len; i++) {
                arr[i] = element.read(buf, offset + i * element.byteSize);
            }
            return arr;
        },
        byteSize,
        description: `Array<${element.description}>(${len})`,
    };
}
exports.fixedSizeArray = fixedSizeArray;
/**
 * A De/Serializer for raw {@link Buffer}s that just copies/reads the buffer bytes
 * to/from the provided buffer.
 *
 * @param bytes the byte size of the buffer to de/serialize
 * @category beet/collection
 */
function fixedSizeBuffer(bytes) {
    return {
        write: function (buf, offset, value) {
            value.copy(buf, offset, 0, bytes);
        },
        read: function (buf, offset) {
            return buf.slice(offset, offset + bytes);
        },
        byteSize: bytes,
        description: `Buffer(len)`,
    };
}
exports.fixedSizeBuffer = fixedSizeBuffer;
/**
 * A De/Serializer for {@link Uint8Array}s that just copies/reads the array bytes
 * to/from the provided buffer.
 *
 * @category beet/collection
 */
function fixedSizeUint8Array(len) {
    const arrayBufferBeet = fixedSizeBuffer(len);
    return {
        write: function (buf, offset, value) {
            const valueBuf = Buffer.from(value);
            arrayBufferBeet.write(buf, offset, valueBuf);
        },
        read: function (buf, offset) {
            const arrayBuffer = arrayBufferBeet.read(buf, offset);
            return Uint8Array.from(arrayBuffer);
        },
        byteSize: len,
        description: `Uint8Array(len)`,
    };
}
exports.fixedSizeUint8Array = fixedSizeUint8Array;
/**
 * Maps collections beet exports to metadata which describes in which package it
 * is defined as well as which TypeScript type is used to represent the
 * deserialized value in JavaScript.
 *
 * @category TypeDefinition
 */
// prettier-ignore
exports.collectionsTypeMap = {
    string: { beet: 'fixedSizeUtf8String', sourcePack: types_2.BEET_PACKAGE, ts: 'string', arg: types_1.BEET_TYPE_ARG_LEN },
    Array: { beet: 'fixedSizeArray', sourcePack: types_2.BEET_PACKAGE, ts: 'Array', arg: types_1.BEET_TYPE_ARG_LEN },
    Buffer: { beet: 'fixedSizeBuffer', sourcePack: types_2.BEET_PACKAGE, ts: 'Buffer', arg: types_1.BEET_TYPE_ARG_LEN },
    Uint8Array: { beet: 'fixedSizeUint8Array', sourcePack: types_2.BEET_PACKAGE, ts: 'Uint8Array', arg: types_1.BEET_TYPE_ARG_LEN }
};
//# sourceMappingURL=collections.js.map